@@ -1,3 +1,24 @@
+/*
+ * This file is part of PowerDNS or dnsdist.
+ * Copyright -- PowerDNS.COM B.V. and its contributors
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of version 2 of the GNU General Public License as
+ * published by the Free Software Foundation.
+ *
+ * In addition, for the avoidance of any doubt, permission is granted to
+ * link this program with OpenSSL and to (re)distribute the binaries
+ * produced as the result of such linking.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+ */
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
@@ -104,6 +125,18 @@ try
             str<<base<<"cache-ttl-too-shorts" << " " << cache->getTTLTooShorts() << " " << now << "\r\n";
           }
         }
+
+        {
+          WriteLock wl(&g_qcount.queryLock);
+          std::string qname;
+          for(auto &record: g_qcount.records) {
+            qname = record.first;
+            boost::replace_all(qname, ".", "_");
+            str<<"dnsdist.querycount." << qname << ".queries " << record.second << " " << now << "\r\n";
+          }
+          g_qcount.records.clear();
+        }
+
         const string msg = str.str();
 
         int ret = waitForRWData(s.getHandle(), false, 1 , 0);
@@ -112,11 +145,7 @@ try
           continue;
         }
         s.setBlocking();
-        ret=writen2(s.getHandle(), msg.c_str(), msg.size());
-        if(ret < 0)
-          warnlog("Error writing carbon data to %s: %s", server.toStringWithPort(), strerror(errno));
-        if(ret==0)
-          warnlog("EOF writing carbon data to %s", server.toStringWithPort());
+        writen2(s.getHandle(), msg.c_str(), msg.size());
       }
       catch(std::exception& e) {
         warnlog("Problem sending carbon data: %s", e.what());
