@@ -1,30 +1,31 @@
 /*
-    PowerDNS Versatile Database Driven Nameserver
-    Copyright (C) 2013 - 2015  PowerDNS.COM BV
-
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License version 2
-    as published by the Free Software Foundation
-
-    Additionally, the license of this program contains a special
-    exception which allows to distribute the program in binary form when
-    it is linked against OpenSSL.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
-*/
-
+ * This file is part of PowerDNS or dnsdist.
+ * Copyright -- PowerDNS.COM B.V. and its contributors
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of version 2 of the GNU General Public License as
+ * published by the Free Software Foundation.
+ *
+ * In addition, for the avoidance of any doubt, permission is granted to
+ * link this program with OpenSSL and to (re)distribute the binaries
+ * produced as the result of such linking.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+ */
 #include "dnsdist.hh"
 #include "dnsdist-ecs.hh"
+#include "dnsparser.hh"
 #include "ednsoptions.hh"
 #include "dolog.hh"
 #include "lock.hh"
+#include "gettime.hh"
 #include <thread>
 #include <atomic>
 
@@ -50,12 +51,19 @@ static int setupTCPDownstream(shared_ptr<DownstreamState> ds)
 {  
   vinfolog("TCP connecting to downstream %s", ds->remote.toStringWithPort());
   int sock = SSocket(ds->remote.sin4.sin_family, SOCK_STREAM, 0);
-  if (!IsAnyAddress(ds->sourceAddr)) {
-    SSetsockopt(sock, SOL_SOCKET, SO_REUSEADDR, 1);
-    SBind(sock, ds->sourceAddr);
+  try {
+    if (!IsAnyAddress(ds->sourceAddr)) {
+      SSetsockopt(sock, SOL_SOCKET, SO_REUSEADDR, 1);
+      SBind(sock, ds->sourceAddr);
+    }
+    SConnect(sock, ds->remote);
+    setNonBlocking(sock);
+  }
+  catch(const std::runtime_error& e) {
+    /* don't leak our file descriptor if SConnect() (for example) throws */
+    close(sock);
+    throw;
   }
-  SConnect(sock, ds->remote);
-  setNonBlocking(sock);
   return sock;
 }
 
@@ -66,40 +74,59 @@ struct ConnectionInfo
   ClientState* cs;
 };
 
-uint64_t g_maxTCPQueuedConnections{0};
+uint64_t g_maxTCPQueuedConnections{1000};
 void* tcpClientThread(int pipefd);
 
-// Should not be called simultaneously!
 void TCPClientCollection::addTCPClientThread()
 {
-  if (d_numthreads >= d_tcpclientthreads.capacity()) {
-    warnlog("Adding a new TCP client thread would exceed the vector capacity (%d/%d), skipping", d_numthreads.load(), d_tcpclientthreads.capacity());
-    return;
-  }
-
   vinfolog("Adding TCP Client thread");
 
   int pipefds[2] = { -1, -1};
-  if(pipe(pipefds) < 0)
-    unixDie("Creating pipe");
+  if (pipe(pipefds) < 0) {
+    errlog("Error creating the TCP thread communication pipe: %s", strerror(errno));
+    return;
+  }
 
   if (!setNonBlocking(pipefds[1])) {
     close(pipefds[0]);
     close(pipefds[1]);
-    unixDie("Setting pipe non-blocking");
+    errlog("Error setting the TCP thread communication pipe non-blocking: %s", strerror(errno));
+    return;
+  }
+
+  {
+    std::lock_guard<std::mutex> lock(d_mutex);
+
+    if (d_numthreads >= d_tcpclientthreads.capacity()) {
+      warnlog("Adding a new TCP client thread would exceed the vector capacity (%d/%d), skipping", d_numthreads.load(), d_tcpclientthreads.capacity());
+      close(pipefds[0]);
+      close(pipefds[1]);
+      return;
+    }
+
+    try {
+      thread t1(tcpClientThread, pipefds[0]);
+      t1.detach();
+    }
+    catch(const std::runtime_error& e) {
+      /* the thread creation failed, don't leak */
+      errlog("Error creating a TCP thread: %s", e.what());
+      close(pipefds[0]);
+      close(pipefds[1]);
+      return;
+    }
+
+    d_tcpclientthreads.push_back(pipefds[1]);
   }
 
-  d_tcpclientthreads.push_back(pipefds[1]);
   ++d_numthreads;
-  thread t1(tcpClientThread, pipefds[0]);
-  t1.detach();
 }
 
 static bool getNonBlockingMsgLen(int fd, uint16_t* len, int timeout)
 try
 {
   uint16_t raw;
-  int ret = readn2WithTimeout(fd, &raw, sizeof raw, timeout);
+  size_t ret = readn2WithTimeout(fd, &raw, sizeof raw, timeout);
   if(ret != sizeof raw)
     return false;
   *len = ntohs(raw);
@@ -113,7 +140,7 @@ static bool putNonBlockingMsgLen(int fd, uint16_t len, int timeout)
 try
 {
   uint16_t raw = htons(len);
-  int ret = writen2WithTimeout(fd, &raw, sizeof raw, timeout);
+  size_t ret = writen2WithTimeout(fd, &raw, sizeof raw, timeout);
   return ret == sizeof raw;
 }
 catch(...) {
@@ -164,6 +191,7 @@ void* tcpClientThread(int pipefd)
   auto localRulactions = g_rulactions.getLocal();
   auto localRespRulactions = g_resprulactions.getLocal();
   auto localDynBlockNMG = g_dynblockNMG.getLocal();
+  auto localDynBlockSMT = g_dynblockSMT.getLocal();
   auto localPools = g_pools.getLocal();
 #ifdef HAVE_PROTOBUF
   boost::uuids::random_generator uuidGenerator;
@@ -180,18 +208,25 @@ void* tcpClientThread(int pipefd)
       throw std::runtime_error("Error reading from TCP acceptor pipe (" + std::to_string(pipefd) + ") in " + std::string(isNonBlocking(pipefd) ? "non-blocking" : "blocking") + " mode: " + e.what());
     }
 
-    --g_tcpclientthreads->d_queued;
+    g_tcpclientthreads->decrementQueuedCount();
     ci=*citmp;
     delete citmp;    
 
     uint16_t qlen, rlen;
-    string poolname;
     string largerQuery;
     vector<uint8_t> rewrittenResponse;
     shared_ptr<DownstreamState> ds;
+    ComboAddress dest;
+    memset(&dest, 0, sizeof(dest));
+    dest.sin4.sin_family = ci.remote.sin4.sin_family;
+    socklen_t len = dest.getSocklen();
     if (!setNonBlocking(ci.fd))
       goto drop;
 
+    if (getsockname(ci.fd, (sockaddr*)&dest, &len)) {
+      dest = ci.cs->local;
+    }
+
     try {
       for(;;) {
         ds = nullptr;
@@ -229,7 +264,7 @@ void* tcpClientThread(int pipefd)
 
           if (!decrypted) {
             if (response.size() > 0) {
-              sendResponseToClient(ci.fd, reinterpret_cast<char*>(response.data()), response.size());
+              sendResponseToClient(ci.fd, reinterpret_cast<char*>(response.data()), (uint16_t) response.size());
             }
             break;
           }
@@ -257,17 +292,20 @@ void* tcpClientThread(int pipefd)
 	uint16_t qtype, qclass;
 	unsigned int consumed = 0;
 	DNSName qname(query, qlen, sizeof(dnsheader), false, &qtype, &qclass, &consumed);
-	DNSQuestion dq(&qname, qtype, qclass, &ci.cs->local, &ci.remote, (dnsheader*)query, querySize, qlen, true);
+	DNSQuestion dq(&qname, qtype, qclass, &dest, &ci.remote, (dnsheader*)query, querySize, qlen, true);
 #ifdef HAVE_PROTOBUF
         dq.uniqueId = uuidGenerator();
 #endif
 
 	string poolname;
 	int delayMsec=0;
+	/* we need this one to be accurate ("real") for the protobuf message */
+	struct timespec queryRealTime;
 	struct timespec now;
-	clock_gettime(CLOCK_MONOTONIC, &now);
+	gettime(&now);
+	gettime(&queryRealTime, true);
 
-	if (!processQuery(localDynBlockNMG, localRulactions, blockFilter, dq, poolname, &delayMsec, now)) {
+	if (!processQuery(localDynBlockNMG, localDynBlockSMT, localRulactions, blockFilter, dq, poolname, &delayMsec, now)) {
 	  goto drop;
 	}
 
@@ -283,9 +321,6 @@ void* tcpClientThread(int pipefd)
 	  goto drop;
 	}
 
-	if(dq.qtype == QType::AXFR || dq.qtype == QType::IXFR)  // XXX fixme we really need to do better
-	  break;
-
         std::shared_ptr<ServerPool> serverPool = getPool(*localPools, poolname);
         std::shared_ptr<DNSDistPacketCache> packetCache = nullptr;
 	{
@@ -294,12 +329,12 @@ void* tcpClientThread(int pipefd)
 	  packetCache = serverPool->packetCache;
 	}
 
-        if (ds && ds->useECS) {
+        if (dq.useECS && ds && ds->useECS) {
           uint16_t newLen = dq.len;
-          handleEDNSClientSubnet(queryBuffer, dq.size, consumed, &newLen, largerQuery, &ednsAdded, &ecsAdded, ci.remote);
+          handleEDNSClientSubnet(queryBuffer, dq.size, consumed, &newLen, largerQuery, &ednsAdded, &ecsAdded, ci.remote, dq.ecsOverride, dq.ecsPrefixLength);
           if (largerQuery.empty() == false) {
             query = largerQuery.c_str();
-            dq.len = largerQuery.size();
+            dq.len = (uint16_t) largerQuery.size();
             dq.size = largerQuery.size();
           } else {
             dq.len = newLen;
@@ -311,7 +346,7 @@ void* tcpClientThread(int pipefd)
           char cachedResponse[4096];
           uint16_t cachedResponseSize = sizeof cachedResponse;
           uint32_t allowExpired = ds ? 0 : g_staleCacheEntriesTTL;
-          if (packetCache->get(dq, consumed, dq.dh->id, cachedResponse, &cachedResponseSize, &cacheKey, allowExpired)) {
+          if (packetCache->get(dq, (uint16_t) consumed, dq.dh->id, cachedResponse, &cachedResponseSize, &cacheKey, allowExpired)) {
 #ifdef HAVE_DNSCRYPT
             if (!encryptResponse(cachedResponse, &cachedResponseSize, sizeof cachedResponse, true, dnsCryptQuery)) {
               goto drop;
@@ -324,14 +359,29 @@ void* tcpClientThread(int pipefd)
           g_stats.cacheMisses++;
         }
 
-	if(!ds) {
-	  g_stats.noPolicy++;
-	  break;
-	}
+        if(!ds) {
+          g_stats.noPolicy++;
+
+          if (g_servFailOnNoPolicy) {
+            restoreFlags(dh, origFlags);
+            dq.dh->rcode = RCode::ServFail;
+            dq.dh->qr = true;
+
+#ifdef HAVE_DNSCRYPT
+            if (!encryptResponse(queryBuffer, &dq.len, dq.size, true, dnsCryptQuery)) {
+              goto drop;
+            }
+#endif
+            sendResponseToClient(ci.fd, query, dq.len);
+          }
+
+          break;
+        }
 
 	int dsock = -1;
 	if(sockets.count(ds->remote) == 0) {
-	  dsock=sockets[ds->remote]=setupTCPDownstream(ds);
+	  dsock=setupTCPDownstream(ds);
+	  sockets[ds->remote]=dsock;
 	}
 	else
 	  dsock=sockets[ds->remote];
@@ -360,7 +410,8 @@ void* tcpClientThread(int pipefd)
           close(dsock);
           dsock=-1;
           sockets.erase(ds->remote);
-          sockets[ds->remote]=dsock=setupTCPDownstream(ds);
+          dsock=setupTCPDownstream(ds);
+          sockets[ds->remote]=dsock;
           downstream_failures++;
           goto retry;
         }
@@ -378,18 +429,31 @@ void* tcpClientThread(int pipefd)
           close(dsock);
           dsock=-1;
           sockets.erase(ds->remote);
-          sockets[ds->remote]=dsock=setupTCPDownstream(ds);
+          dsock=setupTCPDownstream(ds);
+          sockets[ds->remote]=dsock;
           downstream_failures++;
           goto retry;
         }
 
+        bool xfrStarted = false;
+        bool isXFR = (dq.qtype == QType::AXFR || dq.qtype == QType::IXFR);
+        if (isXFR) {
+          dq.skipCache = true;
+        }
+
+      getpacket:;
+
         if(!getNonBlockingMsgLen(dsock, &rlen, ds->tcpRecvTimeout)) {
 	  vinfolog("Downstream connection to %s died on us phase 2, getting a new one!", ds->getName());
           close(dsock);
           dsock=-1;
           sockets.erase(ds->remote);
-          sockets[ds->remote]=dsock=setupTCPDownstream(ds);
+          dsock=setupTCPDownstream(ds);
+          sockets[ds->remote]=dsock;
           downstream_failures++;
+          if(xfrStarted) {
+            goto drop;
+          }
           goto retry;
         }
 
@@ -405,8 +469,11 @@ void* tcpClientThread(int pipefd)
         readn2WithTimeout(dsock, answerbuffer, rlen, ds->tcpRecvTimeout);
         char* response = answerbuffer;
         uint16_t responseLen = rlen;
-        --ds->outstanding;
-        outstanding = false;
+        if (outstanding) {
+          /* might be false for {A,I}XFR */
+          --ds->outstanding;
+          outstanding = false;
+        }
 
         if (rlen < sizeof(dnsheader)) {
           break;
@@ -421,16 +488,16 @@ void* tcpClientThread(int pipefd)
         }
 
         dh = (struct dnsheader*) response;
-        DNSQuestion dr(&qname, qtype, qclass, &ci.cs->local, &ci.remote, dh, responseSize, responseLen, true);
+        DNSResponse dr(&qname, qtype, qclass, &dest, &ci.remote, dh, responseSize, responseLen, true, &queryRealTime);
 #ifdef HAVE_PROTOBUF
         dr.uniqueId = dq.uniqueId;
 #endif
-        if (!processResponse(localRespRulactions, dr)) {
+        if (!processResponse(localRespRulactions, dr, &delayMsec)) {
           break;
         }
 
 	if (packetCache && !dq.skipCache) {
-	  packetCache->insert(cacheKey, qname, qtype, qclass, response, responseLen, true, dh->rcode == RCode::ServFail);
+	  packetCache->insert(cacheKey, qname, qtype, qclass, response, responseLen, true, dh->rcode);
 	}
 
 #ifdef HAVE_DNSCRYPT
@@ -442,13 +509,25 @@ void* tcpClientThread(int pipefd)
           break;
         }
 
+        if (isXFR && dh->rcode == 0 && dh->ancount != 0) {
+          if (xfrStarted == false) {
+            xfrStarted = true;
+            if (getRecordsOfTypeCount(response, responseLen, 1, QType::SOA) == 1) {
+              goto getpacket;
+            }
+          }
+          else if (getRecordsOfTypeCount(response, responseLen, 1, QType::SOA) == 0) {
+            goto getpacket;
+          }
+        }
+
         g_stats.responses++;
         struct timespec answertime;
-        clock_gettime(CLOCK_MONOTONIC, &answertime);
+        gettime(&answertime);
         unsigned int udiff = 1000000.0*DiffTime(now,answertime);
         {
           std::lock_guard<std::mutex> lock(g_rings.respMutex);
-          g_rings.respRing.push_back({answertime,  ci.remote, qname, dq.qtype, (unsigned int)udiff, (unsigned int)responseLen, *dq.dh, ds->remote});
+          g_rings.respRing.push_back({answertime, ci.remote, qname, dq.qtype, (unsigned int)udiff, (unsigned int)responseLen, *dh, ds->remote});
         }
 
         largerQuery.clear();
@@ -485,9 +564,9 @@ void* tcpAcceptorThread(void* p)
 
   auto acl = g_ACL.getLocal();
   for(;;) {
-    ConnectionInfo* ci;
+    bool queuedCounterIncremented = false;
+    ConnectionInfo* ci = nullptr;
     try {
-      ci=0;
       ci = new ConnectionInfo;
       ci->cs = cs;
       ci->fd = -1;
@@ -497,12 +576,12 @@ void* tcpAcceptorThread(void* p)
 	g_stats.aclDrops++;
 	close(ci->fd);
 	delete ci;
-	ci=0;
+	ci=nullptr;
 	vinfolog("Dropped TCP connection from %s because of ACL", remote.toStringWithPort());
 	continue;
       }
 
-      if(g_maxTCPQueuedConnections > 0 && g_tcpclientthreads->d_queued >= g_maxTCPQueuedConnections) {
+      if(g_maxTCPQueuedConnections > 0 && g_tcpclientthreads->getQueuedCount() >= g_maxTCPQueuedConnections) {
         close(ci->fd);
         delete ci;
         ci=nullptr;
@@ -515,10 +594,12 @@ void* tcpAcceptorThread(void* p)
       ci->remote = remote;
       int pipe = g_tcpclientthreads->getThread();
       if (pipe >= 0) {
+        queuedCounterIncremented = true;
         writen2WithTimeout(pipe, &ci, sizeof(ci), 0);
       }
       else {
-        --g_tcpclientthreads->d_queued;
+        g_tcpclientthreads->decrementQueuedCount();
+        queuedCounterIncremented = false;
         close(ci->fd);
         delete ci;
         ci=nullptr;
@@ -529,6 +610,10 @@ void* tcpAcceptorThread(void* p)
       if(ci && ci->fd >= 0) 
 	close(ci->fd);
       delete ci;
+      ci = nullptr;
+      if (queuedCounterIncremented) {
+        g_tcpclientthreads->decrementQueuedCount();
+      }
     }
     catch(...){}
   }
@@ -541,7 +626,7 @@ bool getMsgLen32(int fd, uint32_t* len)
 try
 {
   uint32_t raw;
-  int ret = readn2(fd, &raw, sizeof raw);
+  size_t ret = readn2(fd, &raw, sizeof raw);
   if(ret != sizeof raw)
     return false;
   *len = ntohl(raw);
@@ -557,7 +642,7 @@ bool putMsgLen32(int fd, uint32_t len)
 try
 {
   uint32_t raw = htonl(len);
-  int ret = writen2(fd, &raw, sizeof raw);
+  size_t ret = writen2(fd, &raw, sizeof raw);
   return ret==sizeof raw;
 }
 catch(...) {
